/*                                                              */
/* weird7.c - C code for Weird-7 Biped Robot                    */
/* Copyright 2003 (c) by Kazuhisa Terasaki all rights reserved. */
/* weird-7@kaduhi.com  http://www.kaduhi.com/weird-7            */
/*                                                              */
/* Version 1.04  08/28/2003                                     */


#include "3664f.h"
#include "machine.h"

const unsigned char g_dummy[] = { 0x00, 0x00 };

extern unsigned char *SERVO_BIT_PATTERNS;
extern unsigned char *SAMPLE_POINT_TABLE;

unsigned int	servo_time_array[16];
unsigned int	receivedServoTimeArray[16];
unsigned char	g_nCurrServo;
unsigned int	*g_pCurrServoTime;
unsigned char	*g_pCurrServoBitPattern;
unsigned int	g_ADC_array[8];
unsigned char	*g_pCurrentSampleTarget;
unsigned int	g_nADThreshold;				/* 10bits from MSB, sample = ((*ADC[15:6] + g_nADThreshold) >= 0x10000) ? Hi : Lo; */
unsigned char	g_nCurrentSampleTime;
unsigned char	g_nPrevSample;
unsigned char	g_nSSR;
unsigned char	g_nRSR;
unsigned char	g_nRDR;

void init_io(void)
{
	unsigned char uc;

	/* Initialize Port1 */
	IO.PCR1 = 0xA0;			/* OUT: P17, P15 */
/*	IO.PMR1.BYTE = 0x0C;*/	/* Use P22 as I/O */
	IO.PMR1.BYTE = 0x0E;	/* Use P22 as TXD */

	/* Initialize Port2 */
/*	IO.PCR2 = 0x05;*/			/* OUT: P22, P20 */
	IO.PCR2 = 0x01;			/* OUT: P20 */

	/* Initialize Port5 */
/*	IO.PCR5 = 0x80;*/			/* OUT: P57 (2.5V) */
/*	IO.PDR5.BYTE = 0x80;*/

	/* Initialize Port7 */
	IO.PCR7 = 0x50;			/* OUT: P76, P74 */

	/* Initialize Port8 */
	IO.PCR8 = 0x55;			/* OUT: P86, P84, P82, P80 */

	/* Initialize TimerW */
	TW.TMRW.BYTE = 0x48;	/* Disable timer */
	TW.TCRW.BYTE = 0x80;	/* CCLR=1, Choose 1/1 CLOCK (16MHz) as TimerW clock */
	TW.TSRW.BYTE = 0x70;	/* Clear status flags (OVF, IMFD, IMFC, IMFB, IMFA) */
	TW.TIOR0.BYTE = 0x88;	/* GRB=Output Compare Register, GRA=Output Compare Register */
	TW.TIOR1.BYTE = 0x88;	/* GRD=Output Compare Register, GRC=Output Compare Register */
	TW.TCNT = 0;			/* Reset counter */
	TW.GRB = 24000;			/* 24000 = 1.5mS */
	TW.GRA = 40000;			/* 40000 = 2.5mS */
	TW.TIERW.BYTE = 0x73;	/* Enable IMFB/IMFA Interrupt */

	/* Initialize SCI3 */
	SCI3.SMR.BYTE = 0x00;	/* Choose 1/1 CLOCK (16MHz) as SCI3 clock */
	SCI3.SCR3.BYTE = 0x00;	/* TE=0, RE=0, CKE1=0, CKE0=0 */
	uc = SCI3.SSR.BYTE;		/* Dummy read */
	SCI3.SSR.BYTE = 0x00;	/* Clear status flags */
/*	SCI3.BRR = 103;*/			/* 4800bps */
	SCI3.BRR = 12;			/* 38400bps */
	SCI3.SCR3.BYTE = 0x30;	/* TE=1, RE=1 */

	/* Initialize AD */
	AD.CR.BYTE = 0x00;	/* TRGE=0 */
	AD.CSR.BYTE = 0x5B;	/* ADF=0, ADIE=1, ADST=0, SCAN=1, CKS=1, CH=011 */
}

void init_memory(void)
{
	unsigned int i;

	for (i = 0; i < 10; i++) {
		servo_time_array[i] = 0;
		receivedServoTimeArray[i] = 0;
		
	}
}

void main(void)
{
	unsigned char uc;
	unsigned char uc1;
	unsigned char uc2;
	unsigned char uc3;
	unsigned char nRcvCnt;
	unsigned int  wRcvWord;
	unsigned char nSndCnt;
	unsigned int  wSndWord;
	unsigned int  state;
	unsigned int  nADCMaxValue;
	unsigned int  nPacketReceivedCnt;

	init_io();
	init_memory();

	g_nCurrServo = 9;
	g_pCurrServoTime = servo_time_array;
	g_pCurrServoBitPattern = SERVO_BIT_PATTERNS;

	g_pCurrentSampleTarget = SAMPLE_POINT_TABLE;
	g_nCurrentSampleTime = 0;
	g_nSSR = 0x00;
	g_nADThreshold = 0xFF00;
	nADCMaxValue = 0;

	TW.TMRW.BYTE = 0xC8;	/* Enable timer */
	AD.CSR.BYTE |= 0x20;	/* A/D start */
	set_imask_ccr(0);		/* Enable interrupt */

	nRcvCnt = 0;
	nSndCnt = 0;
	nPacketReceivedCnt = 0;

	state = 0;
	while (1)
	{
		/* Receive data from PC */
		if ((SCI3.SSR.BYTE & 0x78) == 0x40)
		{
			/* No error & received data */
			uc3 = SCI3.RDR;
			SCI3.SSR.BYTE &= 0x87;		/* Clear SCI3 status flags */
			
/*			switch (uc3) {
				case '1':
					TW.TMRW.BYTE = 0xC8;*/	/* Enable timer */
/*					break;
				case '0':
					TW.TMRW.BYTE = 0x48;*/	/* Disable timer */
/*					break;
			}
*/		}
		else if (SCI3.SSR.BYTE & 0x38)
		{
			/* Error */
			SCI3.SSR.BYTE &= 0x87;		/* Clear SCI3 status flags */
		}
		
		if ((g_nSSR & 0x78) == 0x40) {
			uc = g_nRDR;
			g_nSSR &= 0x87;

			if (uc == 0xFF)		/* Start byte? */
			{
				nRcvCnt = 0;
			}
			else
			{
				if ((nRcvCnt & 0x01) == 0)		/* Hi value? */
				{
					/* Hi value */
					wRcvWord = (uc << 8);
				}
				else
				{
					/* Lo value */
					wRcvWord |= uc;
					receivedServoTimeArray[nRcvCnt >> 1] = wRcvWord;
				}

				if (++nRcvCnt >= 20)
				{
					if (nRcvCnt == 20) {
/*						wRcvWord = 0;
						for (uc = 0; uc < 9; uc++) {
							wRcvWord += receivedServoTimeArray[uc];
						}
						if ((wRcvWord == receivedServoTimeArray[9]) || ((wRcvWord & 0xFEEF) == receivedServoTimeArray[9]))
*/						{
							nPacketReceivedCnt++;
							for (uc = 0; uc < 10; uc++) {
								servo_time_array[uc] = receivedServoTimeArray[uc];
							}
						}
					}
					nRcvCnt = 22;
				}
			}
		}
		else if (g_nSSR & 0x38) {
			g_nSSR &= 0x87;
			nRcvCnt = 22;
		}

		if (SCI3.SSR.BYTE & 0x80) {
			uc1 = ' ';
			switch (state) {
				case 0:
					wSndWord = servo_time_array[nSndCnt];
					uc1 = (((wSndWord >> 12) & 0x000F) + '0');
					break;
				case 1:
					uc1 = (((wSndWord >>  8) & 0x000F) + '0');
					break;
				case 2:
					uc1 = (((wSndWord >>  4) & 0x000F) + '0');
					break;
				case 3:
					uc1 = (((wSndWord >>  0) & 0x000F) + '0');
					break;
				case 10:
					uc1 = 0x0D;
					break;
				case 11:
					uc1 = 0x0A;
					break;
			}
			if (uc1 > '9') uc1 += ('A' - '9' - 1);
			SCI3.TDR = uc1;
			if (++state == 5) {
				state = 0;
				if (++nSndCnt >= 11) {
					nSndCnt = 0;
					state = 10;
				}
			}
			else if (state == 12) {
				state = 0;
			}
		}
		
		servo_time_array[10] = nPacketReceivedCnt;
	}
}

