//
//  FirstViewController.m
//  iPhoRobo
//
//  Created by Kazu Terasaki on 4/15/10.
//  Copyright __MyCompanyName__ 2010. All rights reserved.
//

#import "FirstViewController.h"
#import "MotionScript.h"


@implementation FirstViewController


/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	
	scriptView.delegate = self;
	
	for (UIView *v in self.view.subviews) {
		if ([v isKindOfClass:[UISlider class]]) {
			sliders[v.tag] = (UISlider*)v;
		}
		else if ([v isKindOfClass:[UIButton class]]) {
			if (v.tag >= 100) {
				overlayButtons[v.tag - 100] = (UIButton*)v;
			}
		}
	}
	labels[0] = label0;
	labels[1] = label1;
	labels[2] = label2;
	labels[3] = label3;
}

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


- (void)updateSliderLabel:(NSUInteger)servoID {
	NSInteger absPos = sliders[servoID].value;
	NSInteger relPos = absPos - [gMotionScript getServoCenterPosition:servoID];
	NSString *state = @"off";
	if (servosON) {
		state = @"ON ";
	}
	labels[servoID].text = [NSString stringWithFormat:@"%@ %+5d (%5d)", state, relPos, absPos];
}


- (void)viewWillAppear:(BOOL)animated {
	for (NSUInteger index = 0; index < 4; index++) {
		UISlider *slider = sliders[index];
		slider.value = [gMotionScript getServoCenterPosition:index] + [gMotionScript getServoPosition:index];
		[self updateSliderLabel:index];
		[self.view bringSubviewToFront:overlayButtons[index]];
		overlayButtons[index].enabled = YES;
		slider.alpha = 0.5;
	}
	currentSlider = nil;
}


- (void)inactivateAllSliders {
	for (NSUInteger index = 0; index < 4; index++) {
		UISlider *slider = sliders[index];
		slider.alpha = 0.5;
	}
}


- (void)viewDidAppear:(BOOL)animated {
	[self performSelector:@selector(inactivateAllSliders) withObject:nil afterDelay:(NSTimeInterval)0.1];
}


- (IBAction)sliderValueChanged:(id)sender {
	UISlider *slider = (UISlider*)sender;
	
	NSInteger absPos = slider.value;
	NSInteger mod = (absPos % 10);
	if (mod != 0) {
		if (mod >= 5) {
			absPos += (10 - mod);
		}
		else {
			absPos -= mod;
		}
		slider.value = absPos;
	}
	
	NSUInteger servoID = slider.tag;
	NSInteger relPos = absPos - [gMotionScript getServoCenterPosition:servoID];
	if (servosON) {
		[gMotionScript setServoTorque:1 servoID:servoID];
	}
	else {
		[gMotionScript setServoTorque:0 servoID:servoID];
	}
	[gMotionScript setServoPosition:relPos servoID:servoID];
	
	[self updateSliderLabel:servoID];
}


- (IBAction)overlayButtonPressed:(id)sender {
	UIButton *button = (UIButton*)sender;
	for (NSUInteger i = 0; i < 4; i++) {
		UIButton *b = overlayButtons[i];
		UISlider *s = sliders[i];
		if (b == button) {
			[self.view sendSubviewToBack:b];
			b.enabled = NO;
			s.alpha = 1.0;
			currentSlider = s;
		}
		else {
			[self.view bringSubviewToFront:b];
			b.enabled = YES;
			s.alpha = 0.5;
		}
	}
}


- (IBAction)plusMinusButtonPressed:(id)sender {
	if (! currentSlider) return;
	
	switch (((UIButton*)sender).tag) {
		case 0:	// -100
			currentSlider.value = currentSlider.value - 1000;
			break;
			
		case 1:	// -10
			currentSlider.value = currentSlider.value - 100;
			break;
			
		case 2:	// -1
			currentSlider.value = currentSlider.value - 10;
			break;
		
		case 3:	// center
			currentSlider.value = [gMotionScript getServoCenterPosition:currentSlider.tag];
			break;
			
		case 4:	// +1
			currentSlider.value = currentSlider.value + 10;
			break;
			
		case 5:	// +10
			currentSlider.value = currentSlider.value + 100;
			break;
			
		case 6:	// +100
			currentSlider.value = currentSlider.value + 1000;
			break;
	}
	
	[self sliderValueChanged:currentSlider];
}


- (IBAction)onoffAllButtonPressed:(id)sender {
	UIButton *button = (UIButton*)sender;
	if ([[button titleForState:UIControlStateNormal] isEqualToString:@"All OFF"]) {
		servosON = NO;
	}
	else {
		servosON = YES;
	}
	for (NSInteger i = 0; i < 4; i++) {
		[self sliderValueChanged:sliders[i]];
	}
}


- (IBAction)setCenterButtonPressed:(id)sender {
	if (! currentSlider) return;
	UIAlertView *av = [[UIAlertView alloc] initWithTitle:@"Center Position" message:@"Are you sure to update center position?" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"OK", nil];
	[av show];
	[av release];
}


- (IBAction)editDoneButtonPressed:(id)sender {
	[scriptView resignFirstResponder];
	editDoneButton.hidden = YES;
}


#pragma mark UITextViewDelegate

- (void)textViewDidBeginEditing:(UITextView *)textView {
	editDoneButton.hidden = NO;
}


#pragma mark UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
	if (buttonIndex == 0) return;
	
	if ([alertView.title isEqualToString:@"Center Position"]) {
		// update center position
		if (currentSlider) {
			NSUInteger servoID = currentSlider.tag;
			[gMotionScript setServoCenterPosition:currentSlider.value servoID:servoID];
//			[gMotionScript setServoPosition:0 servoID:servoID];
			[self updateSliderLabel:servoID];
		}
	}
}


@end
