//
//  MotionScript.m
//  iPhoRobo
//
//  Created by Kazu Terasaki on 4/15/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "MotionScript.h"


MotionScript *gMotionScript;


@implementation MotionScript


- (void)loadSettings {
	NSUserDefaults *settings = [NSUserDefaults standardUserDefaults];
	
	for (NSUInteger index = 0; index < NUM_SERVOS; index++) {
		short center = (short)[settings integerForKey:[NSString stringWithFormat:@"CENTER_%d", index]];
		if (center == 0) {
			center = 24000;		// default center position 24000 = 1.5mS
		}
		g_arServo_Center[index] = center;
	}
}


- (void)saveSettings {
	NSUserDefaults *settings = [NSUserDefaults standardUserDefaults];
	
	for (NSUInteger index = 0; index < NUM_SERVOS; index++) {
		[settings setInteger:(NSInteger)g_arServo_Center[index] forKey:[NSString stringWithFormat:@"CENTER_%d", index]];
	}
	
	[settings synchronize];
}


- (id)init {
	if (self = [super init]) {
		gMotionScript = self;
		
		[self loadSettings];
		
		servoController = [[ServoController alloc] init];
		[servoController start];
		
		motions = [[NSMutableArray alloc] initWithCapacity:256];
		
		for (int nServoID = 0; nServoID < NUM_SERVOS; nServoID++)
		{
//			g_arServo_Center[nServoID] = 24000;		// 24000 = 1.5mSec
			g_arServo_Direction[nServoID] = +1;
			g_arServo_Torque[nServoID] = 0;
			
			g_arServo_AccelFlag[nServoID] = AccelMode_Accel;
//			g_arServo_Current[nServoID] = 0;
			g_arServo_Target[nServoID] = 0;
			g_arServo_Delta[nServoID] = 0;
			g_arServo_DeltaPlus[nServoID] = 0;
			g_arServo_Plus[nServoID] = 0;
			g_arServo_Start[nServoID] = 0;
			g_arServo_Dir[nServoID] = 0;
			g_arServo_EndPlus[nServoID] = 0;
			g_arServo_Mid[nServoID] = 0;
		}
		
//		g_arServo_Center[0] = 20358;
//		g_arServo_Center[1] = 18412+2600;
//		g_arServo_Center[2] = 27534;
//		g_arServo_Center[3] = 28089;
		
		// start periodic timer
		accelTimer = [NSTimer scheduledTimerWithTimeInterval:0.02 target:self selector:@selector(accelTimerProc:) userInfo:self repeats:YES];
	}
	return self;
}


- (UInt16)getServoCenterPosition:(NSUInteger)servoID {
	if (servoID >= NUM_SERVOS) return 24000;
	
	return g_arServo_Center[servoID];
}


- (BOOL)setServoCenterPosition:(UInt16)centerPos servoID:(NSUInteger)servoID {
	if (servoID >= NUM_SERVOS) return NO;
	
	if ((centerPos < 1000) || (47000 < centerPos)) {
		// out of range
		return NO;
	}
	
	g_arServo_DeltaPlus[servoID] = 0;
	g_arServo_Delta[servoID] = 0;
	g_arServo_Plus[servoID] = g_arServo_EndPlus[servoID] = 0;
	g_arServo_Dir[servoID] = 0;
	g_arServo_Start[servoID] = g_arServo_Target[servoID] = g_arServo_Current[servoID] = ((NSInteger)g_arServo_Target[servoID] + (NSInteger)g_arServo_Center[servoID]) - (NSInteger)centerPos;
	g_arServo_Center[servoID] = centerPos;
	
	[self saveSettings];
	return YES;
}


- (SInt16)getServoPosition:(NSUInteger)servoID {
	if (servoID >= NUM_SERVOS) return 0;
	
	return g_arServo_Target[servoID];
}


- (BOOL)setServoPosition:(SInt16)pos servoID:(NSUInteger)servoID {
	if (servoID >= NUM_SERVOS) return NO;
	
	g_arServo_Target[servoID] = pos;
	return YES;
}


- (BOOL)setServoTorque:(UInt16)torque servoID:(NSUInteger)servoID {
	if (servoID >= NUM_SERVOS) return NO;
	
	g_arServo_Torque[servoID] = torque;
	return YES;
}


- (BOOL)setServoAccelMode:(ServoAccelModeType)accelMode servoID:(NSUInteger)servoID {
	if (servoID >= NUM_SERVOS) return NO;
	
	g_arServo_AccelFlag[servoID] = accelMode;
	return YES;
}


- (BOOL)capture:(NSUInteger)timePos {
	return YES;
}


- (void)accelTimerProc:(NSTimer*)theTimer {
	
	for (NSUInteger nServoID = 0; nServoID < NUM_SERVOS; nServoID++)
	{
		do {
			if (g_arServo_DeltaPlus[nServoID] == 0)
			{
				if (g_arServo_Target[nServoID] == g_arServo_Current[nServoID])
				{
					break;	// do-loop
				}
				else if (g_arServo_AccelFlag[nServoID] == AccelMode_Immediate)
				{
					g_arServo_Current[nServoID] = g_arServo_Target[nServoID];
					break;	// do-loop
				}
				else
				{
					g_arServo_Delta[nServoID] = 0;
					g_arServo_DeltaPlus[nServoID] = SERVO_DELTA_STEP;
					g_arServo_Plus[nServoID] = 0;
					g_arServo_Start[nServoID] = g_arServo_Current[nServoID];
					g_arServo_Dir[nServoID] = ((g_arServo_Target[nServoID] - g_arServo_Start[nServoID]) >= 0) ? +1 : -1;
					g_arServo_EndPlus[nServoID] = (g_arServo_Target[nServoID] - g_arServo_Start[nServoID]) * g_arServo_Dir[nServoID];
					g_arServo_Mid[nServoID] = g_arServo_EndPlus[nServoID] / 2;
				}
			}
			
			g_arServo_Delta[nServoID] += g_arServo_DeltaPlus[nServoID];
			// Delta is always positive number
			if (g_arServo_Delta[nServoID] < SERVO_DELTA_STEP)
			{
				g_arServo_Delta[nServoID] = SERVO_DELTA_STEP;
			}
			
			g_arServo_Plus[nServoID] += g_arServo_Delta[nServoID];
			
			// Reached the target point?
			if (g_arServo_Plus[nServoID] >= g_arServo_EndPlus[nServoID])
			{
				g_arServo_Plus[nServoID] = g_arServo_EndPlus[nServoID];
				g_arServo_DeltaPlus[nServoID] = 0;
			}
			
			// Passing the middle point?
			if (g_arServo_Plus[nServoID] > g_arServo_Mid[nServoID])
			{
				g_arServo_DeltaPlus[nServoID] = 0 - g_arServo_DeltaPlus[nServoID];
				g_arServo_Mid[nServoID] = g_arServo_EndPlus[nServoID];
			}
			
		} while (NO);
		
		g_arServo_Current[nServoID] = g_arServo_Start[nServoID] + g_arServo_Dir[nServoID] * g_arServo_Plus[nServoID];
		if (g_arServo_Torque[nServoID]) {
			[servoController setServoPosition:g_arServo_Center[nServoID] + g_arServo_Direction[nServoID] * g_arServo_Current[nServoID] servoID:nServoID];
		}
		else {
			[servoController setServoPosition:0 servoID:nServoID];
		}
	}
}


@end
