//
//  SoundComm.m
//  soundComm
//
//  Created by Kazu Terasaki on 4/10/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "SoundComm.h"


// audio queue callback routine
void audioQueueOutputCallback(void *inUserData, AudioQueueRef inAQ, AudioQueueBufferRef inBuffer) {
	SoundComm *soundComm = (SoundComm*)inUserData;
	
	UInt32 samplesPerSendingByte = 10 * 2;	// (Start + 8bits + Stop) * 2 samples per bit
	UInt32 bufferSize = inBuffer->mAudioDataBytesCapacity / sizeof(SInt16);
	UInt32 numPackets = (bufferSize / samplesPerSendingByte) * samplesPerSendingByte;
	volatile SInt16 *p = (volatile SInt16*)inBuffer->mAudioData;
	
	// fill audio buffer
	SInt16 hi = 32767;
	SInt16 lo = -32768;
	[soundComm prepareForSending];
	for (int index = 0; index < SEND_DATA_ARRAY_COUNT; index++) {
		UInt16 sendData = [soundComm getSendingValueAtIndex:index];
		
		// send upper byte
		UInt8 data = sendData >> 8;
		*p++ = *p++ = hi;	// start bit
		for (UInt8 mask = 0x80; mask; mask >>= 1) {
			*p++ = *p++ = (data & mask) ? hi : lo;
		}
		*p++ = *p++ = lo;	// stop bit
		
		// send lower byte
		data = sendData & 0x00FF;
		*p++ = *p++ = hi;	// start bit
		for (UInt8 mask = 0x80; mask; mask >>= 1) {
			*p++ = *p++ = (data & mask) ? hi : lo;
		}
		*p++ = *p++ = lo;	// stop bit
		
	}
	
	inBuffer->mPacketDescriptionCount = numPackets;
	inBuffer->mAudioDataByteSize = numPackets * sizeof(SInt16);
	AudioQueueEnqueueBuffer(inAQ, inBuffer, 0, NULL);
}


@implementation SoundComm


- (id)init {
	if (self = [super init]) {
		for (int i =0; i < SEND_DATA_ARRAY_COUNT; i++) {
			sendDataArray[i] = 0xFFFF;
			sendingDataArray[i] = 0xFFFF;
		}
	}
	return self;
}

- (void)dealloc {
	
	[super dealloc];
}


- (void)start {
	audioDesc.mSampleRate = 24000.0;				// 24kHz sampling rate
	audioDesc.mFormatID = kAudioFormatLinearPCM;
	audioDesc.mFormatFlags = 12;					// SInt16
	audioDesc.mBytesPerPacket = 2;
	audioDesc.mFramesPerPacket = 1;
	audioDesc.mBytesPerFrame = 2;
	audioDesc.mChannelsPerFrame = 1;				// Mono
	audioDesc.mBitsPerChannel = 16;
	audioDesc.mReserved = 0;
	AudioQueueNewOutput(&audioDesc, audioQueueOutputCallback, (void*)self, CFRunLoopGetCurrent(), kCFRunLoopCommonModes, 0, &audioQueue);
	
	UInt32 bufferSize = SEND_DATA_ARRAY_COUNT * sizeof(UInt16) * 10 * 2 * 2;	// 24bytes * 10bits * 2samples * sizeof(SInt16)
	for (NSInteger i = 0; i < AUDIO_BUFFER_COUNT; i++) {
		AudioQueueAllocateBuffer(audioQueue, bufferSize, &audioBuffers[i]);
		audioQueueOutputCallback((void*)self, audioQueue, audioBuffers[i]);
	}
	AudioQueueStart(audioQueue, nil);
}


- (void)stop {
	AudioQueueStop(audioQueue, YES);
	AudioQueueDispose(audioQueue, NO);
}


- (void)setValue:(UInt16)value forIndex:(NSUInteger)index {
	if (index < SEND_DATA_ARRAY_COUNT) {
		sendDataArray[index] = value;
	}
}


- (void)prepareForSending {
	for (NSUInteger i = 0; i < SEND_DATA_ARRAY_COUNT; i++) {
		sendingDataArray[i] = sendDataArray[i];
	}
}


- (UInt16)getSendingValueAtIndex:(NSUInteger)index {
	return sendingDataArray[index];
}


@end
